<?php

namespace Drupal\membership\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\entity\Revision\RevisionableContentEntityBase;
use Drupal\membership\Exception\MembershipFeatureNotImplementedException;
use Drupal\membership\Plugin\MembershipProviderManager;
use Drupal\membership\Plugin\SupportsCancellationMembershipProviderInterface;
use Drupal\state_machine\Plugin\Field\FieldType\StateItem;
use Drupal\user\UserInterface;

/**
 * Defines the Membership entity.
 *
 * @ingroup membership
 *
 * @ContentEntityType(
 *   id = "membership",
 *   label = @Translation("Membership"),
 *   label_singular = @Translation("Membership"),
 *   label_plural = @Translation("Memberships"),
 *   label_count = @PluralTranslation(
 *     singular = "@count membership",
 *     plural = "@count memberships",
 *   ),
 *   bundle_label = @Translation("Membership type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\membership\MembershipListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\membership\Form\MembershipForm",
 *       "add" = "Drupal\membership\Form\MembershipForm",
 *       "edit" = "Drupal\membership\Form\MembershipForm",
 *       "delete" = "Drupal\membership\Form\MembershipDeleteForm",
 *     },
 *     "access" = "Drupal\membership\MembershipAccessControlHandler",
 *     "permission_provider" = "Drupal\entity\UncacheableEntityPermissionProvider",
 *     "route_provider" = {
 *       "html" = "Drupal\membership\Routing\AdminHtmlRouteProvider",
 *       "revision" = "Drupal\entity\Routing\RevisionRouteProvider",
 *     },
 *     "storage" = "\Drupal\membership\MembershipStorage",
 *     "views_data" = "\Drupal\membership\MembershipEntityViewsData",
 *   },
 *   base_table = "membership",
 *   revision_table = "membership_revision",
 *   admin_permission = "administer membership entities",
 *   field_ui_base_route = "entity.membership_type.edit_form",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "owner" = "user_id",
 *     "revision" = "vid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log_message"
 *   },
 *   links = {
 *     "canonical" = "/admin/membership/{membership}",
 *     "add-page" = "/admin/membership/add",
 *     "add-form" = "/admin/membership/add/{membership_type}",
 *     "edit-form" = "/admin/membership/{membership}/edit",
 *     "delete-form" = "/admin/membership/{membership}/delete",
 *     "collection" = "/admin/membership",
 *     "revision" = "/admin/membership/{membership}/revisions/{membership_revision}/view",
 *     "version-history" = "/admin/membership/{membership}/revisions",
 *   },
 *   bundle_entity_type = "membership_type"
 * )
 */
class Membership extends RevisionableContentEntityBase implements MembershipInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    if (empty($values['user_id'])) {
      $values['user_id'] = \Drupal::currentUser()->id();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->bundle();
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['type'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Type'))
      ->setDescription(t('The Membership type.'))
      ->setSetting('target_type', 'membership_type')
      ->setRequired(TRUE);
    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Owner'))
      ->setDescription(t('The owner of the Membership entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\node\Entity\Node::getCurrentUserId')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', array(
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => array(
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ),
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    $fields['data'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Data'))
      ->setReadOnly(TRUE)
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE);
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));
    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));
    $fields['state'] = BaseFieldDefinition::create('state')
      ->setLabel(t('State'))
      ->setDescription(t('The membership\'s state.'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'list_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRevisionable(TRUE)
      ->setSetting('workflow_callback', [self::class, 'getWorkflowId']);
    $fields['remote_id'] = BaseFieldDefinition::create('commerce_remote_id')
      ->setLabel('Remote ID')
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
      // This is highly recommended but might not be necessary, e.g. comped.
      ->setRequired(FALSE)
      ->setDisplayConfigurable('form', false)
      ->setDisplayConfigurable('view', false);

    return $fields;
  }

  /**
   * @inheritDoc
   */
  public function cancel() {
    if (!($this->getProviderPlugin() instanceof SupportsCancellationMembershipProviderInterface)) {
      throw new MembershipFeatureNotImplementedException('Membership Cancel method not implemented.');
    }
    $plugin = $this->getProviderPlugin();
    assert($plugin instanceof SupportsCancellationMembershipProviderInterface);
    $plugin->cancel($this);
  }

  /**
   * @inheritDoc
   */
  public function expireNotice() {
    // TODO: Implement expireNotice() method.
    throw new MembershipFeatureNotImplementedException('Membership Expire Notice method not implemented.');
  }

  /**
   * @inheritDoc
   */
  public function extend() {
    // TODO: Implement extend() method.
    throw new MembershipFeatureNotImplementedException('Membership Extend method not implemented.');
  }

  /**
   * @inheritDoc
   */
  public function isActive() {
    return str_starts_with($this->getState()->getId(), 'active');
  }

  /**
   * @inheritDoc
   */
  public function getState(): StateItem {
    return $this->get('state')->first();
  }

  /**
   * @inheritdoc
   */
  static public function getWorkflowId(MembershipInterface $membership) {
    $workflow = MembershipType::load($membership->bundle())->getWorkflowId();
    return $workflow;
  }

  /**
   * {@inheritDoc}
   */
  public function getProviderPlugin() {
    $manager = \Drupal::service('plugin.manager.membership_provider');
    assert($manager instanceof MembershipProviderManager);
    $entityTypeManager = \Drupal::service('entity_type.manager');
    assert($entityTypeManager instanceof EntityTypeManagerInterface);
    $membershipType = $entityTypeManager
      ->getStorage('membership_type')
      ->load($this->bundle());
    assert($membershipType instanceof MembershipTypeInterface);
    return $membershipType->getPlugin();
  }

}
