<?php

declare(strict_types=1);

namespace Drupal\membership;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\entity\UncacheableEntityAccessControlHandler;
use Drupal\membership\Entity\MembershipTypeInterface;
use Drupal\membership\Plugin\MembershipProviderManager;
use Drupal\membership\Plugin\SupportsCreationMembershipProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Membership access control handler.
 */
class MembershipAccessControlHandler extends UncacheableEntityAccessControlHandler implements EntityHandlerInterface {

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   Entity type.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   *
   * @throws \Exception
   */
  public function __construct(
    EntityTypeInterface $entity_type,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected MembershipProviderManager $membershipProviderManager,
  ) {
    parent::__construct($entity_type);
  }

  /**
   * {@inheritDoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.membership_provider')
    );
  }

  public function createAccess($entity_bundle = NULL, AccountInterface $account = NULL, array $context = [], $return_as_object = FALSE) {
    if ($entity_bundle) {
      // Check if the entity bundle allows creation. This may or may not be
      // local, but it could mean we later redirect the user to an offsite
      // creation page.
      $membershipType = $this->entityTypeManager->getStorage('membership_type')
        ->load($entity_bundle);
      assert($membershipType instanceof MembershipTypeInterface);
      $plugin = $this->membershipProviderManager
        ->getDefinition($membershipType->getPluginId());
      assert(!empty($plugin['class']));
      if (!in_array(SupportsCreationMembershipProviderInterface::class, class_implements($plugin['class']))) {
        return AccessResult::forbidden('Membership type does not support creation')
          ->addCacheableDependency($membershipType);
      }
    }
    // @todo This simply delegates to the permissions from the entity API
    //   permissions provider, but these might not sync with the plugin's
    //   capabilities.
    return parent::createAccess($entity_bundle, $account, $context, $return_as_object);
  }

}
